/***************************************************************************
 *   Copyright (C) 2005 by Florian Roth   *
 *   florian@synatic.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "metabar_widget.h"

#include <qsize.h>
#include <qfont.h>
#include <qcolor.h>
#include <qcursor.h>
#include <qhbox.h>
#include <qfile.h>
#include <qobjectlist.h>

#include <klocale.h>
#include <kiconloader.h>
#include <ktrader.h>
#include <kmimetype.h>
#include <krun.h>
#include <kurl.h>
#include <kglobalsettings.h>
#include <kpropertiesdialog.h>
#include <kapplication.h>
#include <dcopref.h>
#include <dcopclient.h>
#include <kinputdialog.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kio/jobclasses.h>
#include <kmessagebox.h>

#include <soundserver.h>
#include <kartsdispatcher.h>

#include <kdebug.h>

MetabarWidget::MetabarWidget(QWidget *parent, const char *name)
    : QWidget( parent, name )
{    
  playobj = 0;
  preview_job = 0;
  
  player_timer = new QTimer();
  connect(player_timer, SIGNAL(timeout()), this, SLOT(updatePlayerStatus()));
  
  dispatcher = new KArtsDispatcher();
  soundServer = Arts::Reference("global:Arts_SoundServerV2");
  factory = new KDE::PlayObjectFactory(soundServer);
  
  config = new KSimpleConfig("metabarrc");
  
  QColor background = KGlobalSettings::baseColor();
  if(!background.isValid()){
    background = QColor(255, 255, 255);
  }
  
  currentPath = new QString("");
  
  actionList.setAutoDelete(true);
  infoList.setAutoDelete(true);
  infoDict.setAutoDelete(true);
    
  dir_watch = new KDirWatch();
  connect(dir_watch, SIGNAL(dirty(const QString&)), this, SLOT(slotUpdateCurrentInfo(const QString&)));
  connect(dir_watch, SIGNAL(created(const QString&)), this, SLOT(slotUpdateCurrentInfo(const QString&)));
  connect(dir_watch, SIGNAL(deleted(const QString&)), this, SLOT(slotDeleteCurrentInfo(const QString&)));
  
  //top box  
  QFrame *top_frame = new QFrame(this);
  top_frame->setMargin(10);
  top_frame->setPaletteBackgroundColor(background);
  top_frame->setFrameShape(QFrame::StyledPanel);
  top_frame->setFrameShadow(QFrame::Raised);
  
  icon = new ClickIcon(top_frame);
  connect(icon, SIGNAL(clicked()), this, SLOT(slotShowPropertiesDialog()));
  
  name_label = new KSqueezedTextLabel(top_frame);
  QFont font = name_label->font();
  font.setBold(true);
  font.setPointSize(14);
  name_label->setFont(font);
  
  path_label = new KSqueezedTextLabel(top_frame);
  
  QHBoxLayout *top_frame_layout = new QHBoxLayout(top_frame, 5, 5);
  QVBoxLayout *top_frame_layout2 = new QVBoxLayout(0);
  
  top_frame_layout2->addWidget(name_label);
  top_frame_layout2->addWidget(path_label);
  
  top_frame_layout->addWidget(icon);
  top_frame_layout->addLayout(top_frame_layout2);
  
  //action box
  action_frame = new QFrame(this);
  action_frame->setPaletteBackgroundColor(background);
  action_frame->setFrameShape(QFrame::StyledPanel);
  action_frame->setFrameShadow(QFrame::Raised);
  
  action_title = new KPushButton(action_frame);
  action_title->setToggleButton(TRUE);
  connect(action_title, SIGNAL(toggled(bool)), this, SLOT(slotExpand(bool)));
  
  font = action_title->font();
  font.setBold(true);
  action_title->setFont(font);
  
  action_frame_layout = new QVBoxLayout(action_frame, 5, 5);
  action_frame_layout->addWidget(action_title);

  
  //info box  
  info_frame = new QFrame(this);
  info_frame->setPaletteBackgroundColor(background);
  info_frame->setFrameShape(QFrame::StyledPanel);
  info_frame->setFrameShadow(QFrame::Raised);
  
  info_title = new KPushButton(i18n("Information"), info_frame);
  info_title->setToggleButton(TRUE);
  connect(info_title, SIGNAL(toggled(bool)), this, SLOT(slotExpand(bool)));
  
  font = info_title->font();
  font.setBold(true);
  info_title->setFont(font);
  
  info_table = new QLabel("", info_frame);
  
  info_frame_layout = new QVBoxLayout(info_frame, 5, 5);
  info_frame_layout->addWidget(info_title);
  info_frame_layout->addWidget(info_table);
  
  //preview box  
  preview_frame = new QFrame(this);
  preview_frame->setPaletteBackgroundColor(background);
  preview_frame->setFrameShape(QFrame::StyledPanel);
  preview_frame->setFrameShadow(QFrame::Raised);
  
  preview_title = new KPushButton(i18n("Preview"), preview_frame);
  preview_title->setToggleButton(TRUE);
  connect(preview_title, SIGNAL(toggled(bool)), this, SLOT(slotExpand(bool)));
  connect(preview_title, SIGNAL(toggled(bool)), this, SLOT(slotCreatePreview(bool)));
  
  font = info_title->font();
  font.setBold(true);
  preview_title->setFont(font);
  
  preview = new QLabel(0);
  preview->setAlignment(Qt::AlignHCenter);
  
  player_frame = new QFrame(preview_frame);
  
  player_start = new KPushButton(player_frame);
  player_start->setPixmap(SmallIcon("player_play"));
  connect(player_start, SIGNAL(clicked()), this, SLOT(startPlayer()));
  
  player_pause = new KPushButton(player_frame);
  player_pause->setPixmap(SmallIcon("player_pause"));
  connect(player_pause, SIGNAL(clicked()), this, SLOT(pausePlayer()));
  
  player_stop = new KPushButton(player_frame);
  player_stop->setPixmap(SmallIcon("player_stop"));
  connect(player_stop, SIGNAL(clicked()), this, SLOT(stopPlayer()));
  
  player_status = new QSlider(Qt::Horizontal, player_frame);
  connect(player_status, SIGNAL(sliderReleased()), this, SLOT(seekPlayer()));
  
  video = new KVideoWidget();
  
  preview_stack = new QWidgetStack(preview_frame);
  preview_stack->addWidget(preview);
  preview_stack->addWidget(video);
  preview_stack->raiseWidget(preview);
  
  QHBoxLayout *player_frame_layout2 = new QHBoxLayout();
  player_frame_layout2->addWidget(player_start);
  player_frame_layout2->addWidget(player_pause);
  player_frame_layout2->addWidget(player_stop);
  
  QVBoxLayout *player_frame_layout = new QVBoxLayout(player_frame, 5, 5);
  player_frame_layout->addLayout(player_frame_layout2);
  player_frame_layout->addWidget(player_status);
  
  QVBoxLayout *preview_frame_layout = new QVBoxLayout(preview_frame, 5, 5);
  preview_frame_layout->addWidget(preview_title);
  preview_frame_layout->addWidget(preview_stack);
  preview_frame_layout->addWidget(player_frame);
  
  //main layout
  QVBoxLayout *main_layout = new QVBoxLayout(this, 5, 5);
  main_layout->addWidget(top_frame);
  main_layout->addWidget(action_frame);
  main_layout->addWidget(info_frame);
  main_layout->addWidget(preview_frame);
  main_layout->addItem(new QSpacerItem(10, 10, QSizePolicy::Expanding, QSizePolicy::Expanding));
  
  //load config
  config->setGroup("General");
  action_title->setOn(config->readBoolEntry("CollapseActions"));
  info_title->setOn(config->readBoolEntry("CollapseInfo"));
  preview_title->setOn(config->readBoolEntry("CollapsePreview"));
}

MetabarWidget::~MetabarWidget(){
  config->setGroup("General");
  config->writeEntry("CollapseActions", action_title->isOn());
  config->writeEntry("CollapseInfo", info_title->isOn());
  config->writeEntry("CollapsePreview", preview_title->isOn());
  config->sync();
  
  deletePlayer();
  
  delete playobj;
  delete factory;
  delete player_timer;
  
  delete config;
  delete dir_watch;
  delete currentPath;
  delete currentItems;
}

void MetabarWidget::setFileItems(const KFileItemList &items)
{   
  int count = items.count();
  
  if(count == 0){
    currentItems = 0;
    
    QString url = getCurrentFolder();
    QString path = KURL(url).path();
    
    if(KURL(url) != KURL(*currentPath)){
      KFileItem current(KFileItem::Unknown, KFileItem::Unknown, KURL(*currentPath), true);
      
      if(dir_watch->contains(*currentPath)){
        if(current.isDir()){
          dir_watch->removeDir(*currentPath);
        }
        else{
          dir_watch->removeFile(*currentPath);
        }
      }
      
      const KFileItem item(KFileItem::Unknown, KFileItem::Unknown, KURL(url), true);
      
      if(item.isDir()){
        dir_watch->addDir(item.url().path());
      }
      else{
        dir_watch->addFile(item.url().path());
      }
          
      *currentPath = path;     
        
      updateSingleInfo(item);
    }
  }
  
  else if(count == 1){
    currentItems = 0;
    
    if(items.getFirst()->url() != KURL("") && items.getFirst()->url() != KURL(*currentPath)){
      KFileItem item(KFileItem::Unknown, KFileItem::Unknown, KURL(*currentPath), true);
      
      if(dir_watch->contains(*currentPath)){
        if(item.isDir()){
          dir_watch->removeDir(*currentPath);
        }
        else{
          dir_watch->removeFile(*currentPath);
        }
      }
      
      KFileItem *newItem = items.getFirst();
      
      if(newItem->url().path() == "blank"){
        newItem = new KFileItem(KFileItem::Unknown, KFileItem::Unknown, KURL(getCurrentFolder()), true);
      }
      
      if(newItem->isDir()){
        dir_watch->addDir(newItem->url().path());
      }
      else{
        dir_watch->addFile(newItem->url().path());
      }      
      
      *currentPath = newItem->url().path();      
      updateSingleInfo(*newItem);
    }
  }
  else{
    *currentPath = "";
    
    if(!currentItems || *currentItems != items){
      currentItems = new KFileItemList(items);
      updateMultiInfo(items);
    }
  }
}

void MetabarWidget::updateSingleInfo(const KFileItem &item){  
  setUpdatesEnabled(FALSE);
  
  single_selection = TRUE;
  
  //delete old data
  actionList.clear();
  infoDict.clear();
  infoList.clear();
  runlist.clear();
  
  deletePlayer();
  
  //get new info  
  const KURL url = item.url();
  QString name = item.name();
  name = name.isEmpty() ? i18n("Root Folder") : name;
  
  name_label->setText(name);
  
  path_label->setText(url.path());
  
  icon->setPixmap(item.pixmap(0, KIcon::DefaultState));
  
  if(item.isDir()){
    action_title->setText(i18n("Actions"));
    
    //create folder
    QHBox *createfolder_box = new QHBox(action_frame);
    createfolder_box->setSpacing(5);
        
    QLabel *createfolder_pixmap = new QLabel(createfolder_box);
    createfolder_pixmap->setPixmap(SmallIcon("folder_new"));
        
    KURLLabel *createfolder_label = new KURLLabel("exec://createnewfolder", i18n("Create New Folder"), createfolder_box);
    createfolder_label->setUnderline(false);
    createfolder_label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        
    connect(createfolder_label, SIGNAL(leftClickedURL()), this, SLOT(createFolder()));
        
    if(!action_title->isOn()){
      createfolder_box->show();
    }
        
    action_frame_layout->add(createfolder_box);
    actionList.append(createfolder_box);
    
    //share
    QHBox *share_box = new QHBox(action_frame);
    share_box->setSpacing(5);
        
    QLabel *share_pixmap = new QLabel(share_box);
    share_pixmap->setPixmap(SmallIcon("network"));
        
    KURLLabel *share_label = new KURLLabel("exec://share", i18n("Share"), share_box);
    share_label->setUnderline(false);
    share_label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        
    connect(share_label, SIGNAL(leftClickedURL()), this, SLOT(slotShowSharingDialog()));
        
    if(!action_title->isOn()){
      share_box->show();
    }
        
    action_frame_layout->add(share_box);
    actionList.append(share_box);
    
    //delete
    QHBox *delete_box = new QHBox(action_frame);
    delete_box->setSpacing(5);
        
    QLabel *delete_pixmap = new QLabel(delete_box);
    delete_pixmap->setPixmap(SmallIcon("editdelete"));
        
    KURLLabel *delete_label = new KURLLabel("exec://delete", i18n("Delete"), delete_box);
    delete_label->setUnderline(false);
    delete_label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        
    connect(delete_label, SIGNAL(leftClickedURL()), this, SLOT(slotDelete()));
        
    if(!action_title->isOn()){
      delete_box->show();
    }
        
    action_frame_layout->add(delete_box);
    actionList.append(delete_box);
    
    //rename
    QHBox *rename_box = new QHBox(action_frame);
    rename_box->setSpacing(5);
        
    QLabel *rename_pixmap = new QLabel(rename_box);
    rename_pixmap->setPixmap(SmallIcon("edit"));
        
    KURLLabel *rename_label = new KURLLabel("exec://rename", i18n("Rename"), rename_box);
    rename_label->setUnderline(false);
    rename_label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        
    connect(rename_label, SIGNAL(leftClickedURL()), this, SLOT(slotRename()));
        
    if(!action_title->isOn()){
      rename_box->show();
    }
        
    action_frame_layout->add(rename_box);
    actionList.append(rename_box);
        
    //hide preview
    preview_frame->hide();
  }
  else{
    action_title->setText(i18n("Open with..."));
    
    KTrader::OfferList offers;
    offers = KTrader::self()->query(item.mimetype(), "Type == 'Application' and DesktopEntryName != 'kfmclient' and DesktopEntryName != 'kfmclient_dir' and DesktopEntryName != 'kfmclient_html'");
    if(!offers.isEmpty()){
      int id = 1;
      
      KTrader::OfferList::ConstIterator it = offers.begin();
      
      for(; it != offers.end(); it++){
        QHBox *url_box = new QHBox(action_frame);
        url_box->setSpacing(5);
        
        QLabel *url_pixmap = new QLabel(url_box);
        url_pixmap->setPixmap((*it)->pixmap(KIcon::Small));
        
        QCString nam;
        nam.setNum(id);
        KURLLabel *url_label = new KURLLabel(url.url(), (*it)->name(), url_box, nam.prepend("action_"));
        url_label->setUnderline(false);
        url_label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        
        connect(url_label, SIGNAL(leftClickedURL()), this, SLOT(slotRun()));
                
        runlist[id++] = *it;
        
        if(!action_title->isOn()){
          url_box->show();
        }
        
        action_frame_layout->add(url_box);
        actionList.append(url_box);
      }
    }
    
    if(!preview_frame->isVisible()){
      preview_frame->show();
    }
  }
  
  QString info("<table><tr><td>" + i18n("Type:") + "</td><td>");
  info = info.append((*item.mimeTypePtr()).comment());
  info = info.append("</td></tr><tr><td>" + i18n("Last modified:") + "</td><td>");
  info = info.append(item.timeString(KIO::UDS_MODIFICATION_TIME));
  info = info.append("</td></tr><tr><td>" + i18n("Accessed:") + "</td><td>");
  info = info.append(item.timeString(KIO::UDS_ACCESS_TIME));
  info = info.append("</td></tr><tr><td>" + i18n("Permissions:") + "</td><td>");
  info = info.append(item.permissionsString());
  info = info.append("</td></tr><tr><td>" + i18n("User:") + "</td><td>");
  info = info.append(item.user());
  info = info.append("</td></tr><tr><td>" + i18n("Group:") + "</td><td>");
  info = info.append(item.group());
  info = info.append("</td></tr><tr><td>" + i18n("Size:") + "</td><td>");
  info = info.append(KIO::convertSize(item.size()));
  info = info.append("</td></tr>");
  
  if(item.isLink()){
    info = info.append("<tr><td>" + i18n("Linktarget:") + "</td><td>");
    info = info.append(item.linkDest());
    info = info.append("</td></tr>");
  }
  
  if(!item.isDir()){
    const KFileMetaInfo &metaInfo = item.metaInfo();
    if(metaInfo.isValid()){
      QStringList groups = metaInfo.supportedGroups();
      
      for(QStringList::ConstIterator it = groups.begin(); it != groups.end(); ++it){
        KFileMetaInfoGroup group = metaInfo.group(*it);
        if(group.isValid()){
          KURLLabel *title_label = new KURLLabel("exec://" + group.name(), group.translatedName(), info_frame);
          title_label->setUnderline(FALSE);
          connect(title_label, SIGNAL(leftClickedURL()), this, SLOT(slotPopupInfo()));
          
          QLabel *info_label = new QLabel(info_frame);
          
          infoDict.insert(group.translatedName(), info_label);
          infoList.append(title_label);
          
          info_frame_layout->addWidget(title_label);
          info_frame_layout->addWidget(info_label);
          
          QStringList keys = group.supportedKeys();
          
          QString str("<table>");
        
          for(QStringList::ConstIterator it = keys.begin(); it != keys.end(); ++it){
            const KFileMetaInfoItem metaInfoItem = group.item(*it);
            
            if(metaInfoItem.isValid()){
              str = str.append("<tr><td>" + metaInfoItem.translatedKey() + "</td><td>");
              str = str.append(metaInfoItem.string());
              str = str.append("</td></tr>");
            }
          }
          str = str.append("</table>");
          info_label->setText(str);
          
          if(!info_title->isOn()){
            title_label->show();
          }
        }
      }
    }
  }
  
  info = info.append("</table>");
  
  info_table->setText(info);
  
  if(!preview_title->isOn()){
    killJobs();
    
    if(item.mimetype().startsWith("video/") || item.mimetype().startsWith("audio/")){
      player_status->setValue(0);
      player_frame->show();
      
      playobj = factory->createPlayObject(item.url(), true);

      if (playobj && !playobj->object().isNull()){
        if(item.mimetype().startsWith("video/")){
          isVideo = TRUE;
          video->embed((Arts::VideoPlayObject)Arts::DynamicCast(playobj->object()));
        }
        else{
          isVideo = FALSE;
        }
      }
    }
    else{
      player_frame->hide();
    }
        
    preview->clear();
    preview->setText(i18n("Creating preview"));
    
    preview_stack->raiseWidget(preview);
    preview_stack->show();

    preview_job = KIO::filePreview(KURL::List(item.url()), preview_stack->width(), 0, 0, 0, TRUE, TRUE, 0);
    
    connect(preview_job, SIGNAL(gotPreview(const KFileItem*, const QPixmap&)), this, SLOT(slotSetPreview(const KFileItem*, const QPixmap&)));
    connect(preview_job, SIGNAL(result(KIO::Job*)), this, SLOT(slotGotPreview()));
  }
  
  setUpdatesEnabled(TRUE);
  repaint();
}

void MetabarWidget::updateMultiInfo(const KFileItemList &items){  
  setUpdatesEnabled(FALSE);
  
  single_selection = FALSE;
  
  //delete old data
  actionList.clear();
  infoDict.clear();
  infoList.clear();
  runlist.clear();
  
  deletePlayer();
  
  //get new info  
  action_title->setText(i18n("Actions"));
  
  KIO::filesize_t size = 0;
  int files = 0;
  int dirs  = 0;
  
  for(KFileItemListIterator it(items); it.current(); ++it){
    size += (*it)->size();
    
    if((*it)->isDir()){
      dirs++;
    }
    else{
      files++;
    }
  }
  
  QString name = QString().setNum(items.count()) + " " + i18n("Items");  
  name_label->setText(name);
  
  path_label->setText(QString().setNum(files) + " " + i18n("Files") + ", " + QString().setNum(dirs) + " " + i18n("Folders"));
  
  icon->setPixmap(DesktopIcon("kmultiple"));
  
  //ACTIONS
  //delete old
  QHBox *delete_box = new QHBox(action_frame);
  delete_box->setSpacing(5);
        
  QLabel *delete_pixmap = new QLabel(delete_box);
  delete_pixmap->setPixmap(SmallIcon("editdelete"));
        
  KURLLabel *delete_label = new KURLLabel("exec://delete", i18n("Delete"), delete_box);
  delete_label->setUnderline(false);
  delete_label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        
  connect(delete_label, SIGNAL(leftClickedURL()), this, SLOT(slotDelete()));
        
  if(!action_title->isOn()){
    delete_box->show();
  }
        
  action_frame_layout->add(delete_box);
  actionList.append(delete_box);  
  
  QString info("<table><tr><td>" + i18n("Size:") + "</td><td>");
  info = info.append(KIO::convertSize(size));
  info = info.append("</td></tr></table>");
  
  info_table->setText(info);
  
  killJobs();
  preview->clear();
  preview_stack->hide();
  preview_frame->hide();
  
  setUpdatesEnabled(TRUE);
  repaint();
}

void MetabarWidget::slotUpdateCurrentInfo(const QString &path)
{
  if(single_selection){
    const KFileItem item(KFileItem::Unknown, KFileItem::Unknown, KURL(path), true);
    updateSingleInfo(item);
  }
}

void MetabarWidget::slotDeleteCurrentInfo(const QString &path)
{
  if(single_selection){
    QString url = getCurrentFolder();
    
    if(KURL(url) != KURL(*currentPath)){
      if(dir_watch->contains(*currentPath)){
        dir_watch->removeDir(*currentPath);
      }
      dir_watch->addDir(url);
          
      *currentPath = url;
          
      const KFileItem item(KFileItem::Unknown, KFileItem::Unknown, url, true);
        
      updateSingleInfo(item);
    }
  }
}

void MetabarWidget::slotGotPreview()
{
  preview_job = 0;
  
  if(!preview_title->isOn()){
    if (preview->pixmap()){
      preview_stack->raiseWidget(preview);
      preview_stack->show();
    }
    else{
      preview->clear();
      preview_stack->hide();
    }
  }
}

void MetabarWidget::killJobs()
{
  if(preview_job){
    preview_job->kill();
    preview_job = 0;
  }
}

void MetabarWidget::slotSetPreview(const KFileItem* item, const QPixmap& icon)
{
  preview_stack->resize(icon.size());
  preview->setPixmap(icon);
}

void MetabarWidget::slotCreatePreview(bool toggled)
{
  if(!toggled){
    killJobs();
    
    const KURL url(*currentPath);
    KFileItem item(KFileItem::Unknown, KFileItem::Unknown, url, true);
    
    if(item.mimetype().startsWith("video/") || item.mimetype().startsWith("audio/")){
      player_frame->show();
      
      playobj = factory->createPlayObject(item.url(), true);

      if (playobj && !playobj->object().isNull()){
        
        if(item.mimetype().startsWith("video/")){
          isVideo = TRUE;
          video->embed((Arts::VideoPlayObject)Arts::DynamicCast(playobj->object()));
        }
        else{
          isVideo = FALSE;
        }
      }
    }
    else{
      player_frame->hide();
    }
        
    preview->clear();
    preview->setText(i18n("Creating preview"));
      
    preview_stack->raiseWidget(preview);
    preview_stack->show();
  
    preview_job = KIO::filePreview(KURL::List(url), preview_stack->width(), 0, 0, 0, TRUE, FALSE, 0);
    
    connect(preview_job, SIGNAL(gotPreview(const KFileItem*, const QPixmap&)), this, SLOT(slotSetPreview(const KFileItem*, const QPixmap&)));
    connect(preview_job, SIGNAL(result(KIO::Job*)),this, SLOT(slotGotPreview()));
  }
}

void MetabarWidget::slotRun()
{
  const QObject *sender = this->sender();
  if(sender->inherits("KURLLabel")){
    const KURLLabel *label = static_cast<const KURLLabel*>(sender);
    
    QCString sender_name = sender->name();
    int id = sender_name.mid(sender_name.find("_") + 1).toInt();
      
    QMap<int,KService::Ptr>::Iterator it = runlist.find(id);
    if(it != runlist.end()){
      KRun::run( **it, KURL::List(KURL(label->url())));
      return; 
    }
  }
}

void MetabarWidget::slotExpand(bool collapse)
{
  const QObject *sender = this->sender();
  const QWidget *sender_widget = static_cast<const QWidget*>(sender);
  
  QLayoutIterator it = sender_widget->parentWidget()->layout()->iterator();
  QLayoutItem *child;
  
  while((child = it.current()) != 0){
    QWidget *widget = child->widget();
    
    if(widget != 0 && !widget->inherits("KPushButton")){
      if(collapse){
        widget->hide();
      }
      else{
        widget->show();
      }
    }
    ++it;
  }
}

void MetabarWidget::slotShowPropertiesDialog()
{
  if(single_selection){
    if(!currentPath->isNull() && !currentPath->isEmpty()){
      KFileItem *item = new KFileItem(KFileItem::Unknown, KFileItem::Unknown, KURL(*currentPath), true);
      new KPropertiesDialog(item);
    }
  }
  else{
    if(currentItems){
      new KPropertiesDialog(*currentItems);
    }
  }
}

void MetabarWidget::slotShowSharingDialog()
{
  if(single_selection){
    if(!currentPath->isNull() && !currentPath->isEmpty()){
      KFileItem *item = new KFileItem(KFileItem::Unknown, KFileItem::Unknown, KURL(*currentPath), true);
      KPropertiesDialog *dialog = new KPropertiesDialog(item);
      dialog->showFileSharingPage();
    }
  }
}

void MetabarWidget::createFolder()
{
  QString name = i18n("New Folder");
  name = KInputDialog::getText(i18n("Create New Folder"), i18n("Enter folder name:"), name);
  
  if(!name.isEmpty()){
    KURL url(getCurrentFolder());
    QString path = url.path();
    
    if(!path.endsWith("/")){
      path += "/";
    }
    
    path += name;
    
    if(QFile::exists(path)){
      KMessageBox::error(this, i18n("A folder named %1 already exists.").arg(name), i18n("Error"));
    }
    else{
      KGlobal::dirs()->makeDir(path);
    }
  }
}

void MetabarWidget::slotDelete()
{
  KURL::List urls;
  QStringList strings;
  
  if(currentItems){
    for(KFileItemListIterator it(*currentItems); it.current(); ++it){
      urls.append((*it)->url());
      strings.append((*it)->url().path());
    }
    
    if(KMessageBox::warningContinueCancelList(this, i18n("Are you sure you want delete these files?"), strings, i18n("Delete")) == KMessageBox::Continue){
      KIO::del(urls);
    }
  }
  else if(currentPath){
    urls.append(KURL(*currentPath));
    strings.append(*currentPath);
        
    if(KMessageBox::warningContinueCancelList(this, i18n("Are you sure you want delete these files?") ,strings, i18n("Delete")) == KMessageBox::Continue){
      KIO::del(urls);
    }
  }
}

void MetabarWidget::slotRename()
{
  if(currentPath){
    KURL src(*currentPath);
    
    KFileItem item(KFileItem::Unknown, KFileItem::Unknown, src, true);
    QString name = item.name();
    name = KInputDialog::getText(i18n("Rename"), i18n("Enter new name:"), name);
  
    if(!name.isEmpty()){
      KURL url(getCurrentFolder());
      QString path = url.path();
      if(!path.endsWith("/")){
        path += "/";
      }
      
      path += name;
      
      
      bool overwrite = false;
      
      if(QFile(path).exists()){
        if(KMessageBox::warningContinueCancel(this, i18n("The file already exists.\nDo you want to overwrite it?"), i18n("File exists")) == KMessageBox::Continue){
          overwrite = true;
        }
        else{
          return;
        }
      }
      
      KURL dest(path);
      KIO::rename(src, dest, overwrite);
    }
  }
}

void MetabarWidget::slotPopupInfo()
{
  const KURLLabel *title = static_cast<const KURLLabel*>(sender());
  QLabel *label = infoDict[title->text()];
  
  if(label){
    if(label->isVisible()){
      label->hide();
    }
    else{
      label->show();
    }
  }
}

void MetabarWidget::startPlayer()
{  
  if(playobj &&
     !playobj->object().isNull() &&
     playobj->state() != Arts::posPlaying)
  {
    
    setUpdatesEnabled(FALSE);
    
    if(isVideo){
      video->setFixedSize(preview->size());
      preview_stack->setMaximumHeight(preview->height());
      preview_stack->raiseWidget(video);
    }
    
    if(playobj->state() == Arts::posIdle){
      player_status->setValue(0);
      player_status->setMaxValue(playobj->overallTime().seconds);
    }
    
    player_timer->start(1000);
    playobj->play();
    
    setUpdatesEnabled(TRUE);
  }
}

void MetabarWidget::pausePlayer()
{  
  if(playobj &&
     !playobj->object().isNull() &&
     playobj->state() != Arts::posPaused)
  {
    
    if(player_timer->isActive()){
      player_timer->stop();
    }
    
    playobj->pause();
  }
}

void MetabarWidget::stopPlayer()
{
  if(player_timer->isActive()){
    player_timer->stop();
  }
  player_status->setValue(0);
  
  if(playobj && 
    !playobj->object().isNull() &&
     playobj->state() != Arts::posIdle)
  {
    
    setUpdatesEnabled(FALSE);
    
    playobj->halt();
    
    video->setFixedSize(0, 0);
  
    if(isVideo){
      preview_stack->raiseWidget(preview);
    }
      
    setUpdatesEnabled(TRUE);
  }
}

void MetabarWidget::deletePlayer()
{  
  stopPlayer();
  
  playobj = 0;
  
  if(preview_stack->isVisible()){
    preview_stack->hide();
  }
}

void MetabarWidget::updatePlayerStatus()
{
  if(playobj && !playobj->object().isNull()){
    player_status->setValue(playobj->currentTime().seconds);
    
    if(playobj->currentTime().ms == playobj->overallTime().ms){
      player_timer->stop();
      stopPlayer();
    }
  }
}

void MetabarWidget::seekPlayer()
{
  if(playobj && !playobj->object().isNull()){    
    playobj->seek(Arts::poTime(player_status->value(), 0, 0, ""));
  }
}

QString MetabarWidget::getCurrentFolder()
{
  DCOPRef ref(kapp->dcopClient()->appId(), this->topLevelWidget()->name());
  DCOPReply reply = ref.call("currentURL()");
    
  if (reply.isValid()) {
    QString url;
    reply.get(url, "QString");
      
    if(!url.isNull() && !url.isEmpty()){
      return url;
    }
  }
  return 0;
}

void MetabarWidget::showEvent(QShowEvent *event)
{
  *currentPath = QString("");
  
  const KFileItem *item = new KFileItem(KFileItem::Unknown, KFileItem::Unknown, KURL(getCurrentFolder()), true);
  KFileItemList list;
  list.append(item);
  
  setFileItems(list);
}

#include "metabar_widget.moc"
